package cache

import "errors"

var ItemNotFoundError = errors.New("item not found in cache")

// A Cache is a memory safe event-driven key value store. Items are stored
// using a unique key and can only be retrieved with that key. Use ListKeys()
// to get a list of all keys currently in the cache. The implementation of
// the Cache updates in the background and does not allow modification from
// the outside.
type Cache interface {
	GetItem(key string) (interface{}, error)
	GetItemsByNamespace(namespace string) ([]interface{}, error)
	ListKeys() ([]string, error)
}
