#!/usr/bin/env python3

import argparse
import os

import requests

PROJECT_ID = os.getenv("CI_PROJECT_ID", "18")
GITLAB_URL = f"https://gitlab.greeson.xyz/api/v4/projects/{PROJECT_ID}/releases"
HEADERS = {"JOB-TOKEN": os.getenv("CI_JOB_TOKEN")}
MESSAGE = os.getenv("CI_COMMIT_DESCRIPTION", "New update for go-common")
REF = os.getenv("CI_COMMIT_REF_NAME", "main")


def get_latest_release() -> str:
    """Get the latest release"""
    url = f"{GITLAB_URL}/permalink/latest"
    resp = requests.get(url, headers=HEADERS)
    resp.raise_for_status()

    return resp.json()["name"]


def increment_release(release: str) -> str:
    """Increment the release (i.e. 0.1.2 -> 0.1.3)"""
    # Strip the v
    version = release[1:]
    major, minor, rev = version.split(".")
    updated_rev = str(int(int(rev) + 1))
    return f"v{major}.{minor}.{updated_rev}"


def create_release(release: str) -> None:
    """Create a new release"""
    body = {
        "name": release,
        "tag_name": release,
        "tag_message": MESSAGE,
        "description": MESSAGE,
        "ref": REF,
    }
    resp = requests.post(GITLAB_URL, headers=HEADERS, json=body)
    resp.raise_for_status()


if __name__ == "__main__":
    release = get_latest_release()
    print(f"Latest release is {release}")

    new_release = increment_release(release)
    print(f"Creating new release {new_release}")
    create_release(new_release)
