package kubernetes

import (
	"errors"
	"strings"
	"testing"

	"k8s.io/client-go/kubernetes/fake"
)

func TestNewInformerFor(t *testing.T) {
	client = fake.NewSimpleClientset()
	tests := []struct {
		name         string
		resourceType int
		err          error
	}{
		{
			name:         "test create pod informer successfully",
			resourceType: Pod,
		},
		{
			name:         "test create configmap informer successfully",
			resourceType: ConfigMap,
		},
		{
			name:         "test unsupported resource type",
			resourceType: -1,
			err:          errors.New("unsupported resource type"),
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			_, err := NewInformerFor(tt.resourceType, "foo")
			switch err.(type) {
			case nil:
				if tt.err != nil {
					t.Fatalf("expected error: %v, but got no error", tt.err)
				}

			default:
				if tt.err == nil {
					t.Fatalf("expected no error, but got: %v", err)
				}

				if !strings.Contains(err.Error(), tt.err.Error()) {
					t.Errorf("expected: %v, but got: %v", tt.err, err)
				}
			}
		})
	}
}

func TestGetK8sClient(t *testing.T) {
	client = nil
	c, err := GetK8sClient()
	if c == nil && err == nil {
		t.Errorf("client and error were both nil")
	}

	if c != nil && err != nil {
		t.Errorf("client and error were both not nil")
	}
}
