package mocks

import (
	"context"

	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
	"k8s.io/apimachinery/pkg/runtime/schema"
	"k8s.io/apimachinery/pkg/types"
	"k8s.io/apimachinery/pkg/watch"
	"k8s.io/client-go/dynamic"
)

type MockResourceOperations struct {
	Err error
}

func (m MockResourceOperations) Create(ctx context.Context, obj *unstructured.Unstructured, options metav1.CreateOptions, subresources ...string) (*unstructured.Unstructured, error) {
	return nil, m.Err
}

func (m MockResourceOperations) Update(ctx context.Context, obj *unstructured.Unstructured, options metav1.UpdateOptions, subresources ...string) (*unstructured.Unstructured, error) {
	return nil, m.Err
}

func (m MockResourceOperations) UpdateStatus(ctx context.Context, obj *unstructured.Unstructured, options metav1.UpdateOptions) (*unstructured.Unstructured, error) {
	return nil, m.Err
}

func (m MockResourceOperations) Delete(ctx context.Context, name string, options metav1.DeleteOptions, subresources ...string) error {
	return m.Err
}

func (m MockResourceOperations) DeleteCollection(ctx context.Context, options metav1.DeleteOptions, listOptions metav1.ListOptions) error {
	return m.Err
}

func (m MockResourceOperations) Get(ctx context.Context, name string, options metav1.GetOptions, subresources ...string) (*unstructured.Unstructured, error) {
	return nil, m.Err
}

func (m MockResourceOperations) List(ctx context.Context, opts metav1.ListOptions) (*unstructured.UnstructuredList, error) {
	return nil, m.Err
}

func (m MockResourceOperations) Watch(ctx context.Context, opts metav1.ListOptions) (watch.Interface, error) {
	return nil, m.Err
}

func (m MockResourceOperations) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, options metav1.PatchOptions, subresources ...string) (*unstructured.Unstructured, error) {
	return nil, m.Err
}

func (m MockResourceOperations) Apply(ctx context.Context, name string, obj *unstructured.Unstructured, options metav1.ApplyOptions, subresources ...string) (*unstructured.Unstructured, error) {
	return nil, m.Err
}

func (m MockResourceOperations) ApplyStatus(ctx context.Context, name string, obj *unstructured.Unstructured, options metav1.ApplyOptions) (*unstructured.Unstructured, error) {
	return nil, m.Err
}

type MockResource struct{ Err error }

func (m MockResource) Namespace(namespace string) dynamic.ResourceInterface {
	return &MockResourceOperations{Err: m.Err}
}

func (m MockResource) Create(ctx context.Context, obj *unstructured.Unstructured, options metav1.CreateOptions, subresources ...string) (*unstructured.Unstructured, error) {
	return nil, m.Err
}

func (m MockResource) Update(ctx context.Context, obj *unstructured.Unstructured, options metav1.UpdateOptions, subresources ...string) (*unstructured.Unstructured, error) {
	return nil, m.Err
}

func (m MockResource) UpdateStatus(ctx context.Context, obj *unstructured.Unstructured, options metav1.UpdateOptions) (*unstructured.Unstructured, error) {
	return nil, m.Err
}

func (m MockResource) Delete(ctx context.Context, name string, options metav1.DeleteOptions, subresources ...string) error {
	return m.Err
}

func (m MockResource) DeleteCollection(ctx context.Context, options metav1.DeleteOptions, listOptions metav1.ListOptions) error {
	return m.Err
}

func (m MockResource) Get(ctx context.Context, name string, options metav1.GetOptions, subresources ...string) (*unstructured.Unstructured, error) {
	return nil, m.Err
}

func (m MockResource) List(ctx context.Context, opts metav1.ListOptions) (*unstructured.UnstructuredList, error) {
	return nil, m.Err
}

func (m MockResource) Watch(ctx context.Context, opts metav1.ListOptions) (watch.Interface, error) {
	return nil, m.Err
}

func (m MockResource) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, options metav1.PatchOptions, subresources ...string) (*unstructured.Unstructured, error) {
	return nil, m.Err
}

func (m MockResource) Apply(ctx context.Context, name string, obj *unstructured.Unstructured, options metav1.ApplyOptions, subresources ...string) (*unstructured.Unstructured, error) {
	return nil, m.Err
}

func (m MockResource) ApplyStatus(ctx context.Context, name string, obj *unstructured.Unstructured, options metav1.ApplyOptions) (*unstructured.Unstructured, error) {
	return nil, m.Err
}

type MockClient struct{ Err error }

func (mc MockClient) Resource(resource schema.GroupVersionResource) dynamic.NamespaceableResourceInterface {
	return &MockResource{Err: mc.Err}
}
