package mocks

import (
	"context"
	"testing"

	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/runtime/schema"
	"k8s.io/apimachinery/pkg/types"
)

func TestMockDiscordSession(t *testing.T) {
	session := MockDiscordSession{}
	session.Open()
	session.Close()
	session.AddHandler("foo")
	session.ApplicationCommandCreate("foo", "bar", nil)
	session.InteractionRespond(nil, nil)
	session.ChannelMessageSendComplex("foo", nil)
}

func TestMockK8sClient(t *testing.T) {
	ctx := context.TODO()
	client := MockClient{}

	resource := client.Resource(schema.GroupVersionResource{
		Group:    "foo",
		Version:  "bar",
		Resource: "something",
	})
	resource.Create(ctx, nil, metav1.CreateOptions{})
	resource.Update(ctx, nil, metav1.UpdateOptions{})
	resource.UpdateStatus(ctx, nil, metav1.UpdateOptions{})
	resource.Delete(ctx, "foo", metav1.DeleteOptions{})
	resource.DeleteCollection(ctx, metav1.DeleteOptions{}, metav1.ListOptions{})
	resource.Get(ctx, "foo", metav1.GetOptions{})
	resource.List(ctx, metav1.ListOptions{})
	resource.Watch(ctx, metav1.ListOptions{})
	resource.Patch(ctx, "foo", types.JSONPatchType, []byte("foo"), metav1.PatchOptions{})
	resource.Apply(ctx, "foo", nil, metav1.ApplyOptions{})
	resource.ApplyStatus(ctx, "foo", nil, metav1.ApplyOptions{})

	ns := resource.Namespace("foo")
	ns.Create(ctx, nil, metav1.CreateOptions{})
	ns.Update(ctx, nil, metav1.UpdateOptions{})
	ns.UpdateStatus(ctx, nil, metav1.UpdateOptions{})
	ns.Delete(ctx, "foo", metav1.DeleteOptions{})
	ns.DeleteCollection(ctx, metav1.DeleteOptions{}, metav1.ListOptions{})
	ns.Get(ctx, "foo", metav1.GetOptions{})
	ns.List(ctx, metav1.ListOptions{})
	ns.Watch(ctx, metav1.ListOptions{})
	ns.Patch(ctx, "foo", types.JSONPatchType, []byte("foo"), metav1.PatchOptions{})
	ns.Apply(ctx, "foo", nil, metav1.ApplyOptions{})
	ns.ApplyStatus(ctx, "foo", nil, metav1.ApplyOptions{})
}
