package saltlog

import "fmt"

// The TestLogger is useful for tests in that it tracks calls to its methods.
type TestLogger struct {
	InfoCalls   []string
	DebugCalls  []string
	WarnCalls   []string
	ErrorCalls  []string
	PanicCalls  []string
	InfofCalls  []string
	DebugfCalls []string
	WarnfCalls  []string
	ErrorfCalls []string
	PanicfCalls []string
}

func (t *TestLogger) Info(msg string) {
	t.InfoCalls = append(t.InfoCalls, msg)
}

func (t *TestLogger) Debug(msg string) {
	t.DebugCalls = append(t.DebugCalls, msg)
}

func (t *TestLogger) Warn(msg string) {
	t.WarnCalls = append(t.WarnCalls, msg)
}

func (t *TestLogger) Error(msg string) {
	t.ErrorCalls = append(t.ErrorCalls, msg)
}

func (t *TestLogger) Panic(msg string) {
	panic(msg)
}

func (t *TestLogger) Infof(msg string, args ...interface{}) {
	t.InfofCalls = append(t.InfofCalls, fmt.Sprintf(msg, args...))
}

func (t *TestLogger) Debugf(msg string, args ...interface{}) {
	t.DebugfCalls = append(t.DebugfCalls, fmt.Sprintf(msg, args...))
}

func (t *TestLogger) Warnf(msg string, args ...interface{}) {
	t.WarnfCalls = append(t.WarnfCalls, fmt.Sprintf(msg, args...))
}

func (t *TestLogger) Errorf(msg string, args ...interface{}) {
	t.ErrorfCalls = append(t.ErrorfCalls, fmt.Sprintf(msg, args...))
}

func (t *TestLogger) Panicf(msg string, args ...interface{}) {
	panic(fmt.Sprintf("%s: %s", getTime(), fmt.Sprintf(msg, args...)))
}

func (t *TestLogger) AssertCalledWith(logLevel int, msg string) error {
	switch logLevel {
	case LevelInfo:
		if !stringInList(msg, t.InfoCalls) && !stringInList(msg, t.InfofCalls) {
			return fmt.Errorf("Message '%s' not in Info calls: %v, %v", msg, t.InfoCalls, t.InfofCalls)
		}
	case LevelDebug:
		if !stringInList(msg, t.DebugCalls) && !stringInList(msg, t.DebugfCalls) {
			return fmt.Errorf("Message '%s' not in Debug calls: %v, %v", msg, t.DebugCalls, t.DebugfCalls)
		}
	case LevelWarning:
		if !stringInList(msg, t.WarnCalls) && !stringInList(msg, t.WarnfCalls) {
			return fmt.Errorf("Message '%s' not in Warn calls: %v, %v", msg, t.WarnCalls, t.WarnfCalls)
		}
	case LevelError:
		if !stringInList(msg, t.ErrorCalls) && !stringInList(msg, t.ErrorfCalls) {
			return fmt.Errorf("Message '%s' not in Error calls: %v, %v", msg, t.ErrorCalls, t.ErrorfCalls)
		}
	default:
		return fmt.Errorf("Log Level %d does not exist", logLevel)
	}
	return nil
}

func NewTestLogger() *TestLogger {
	return &TestLogger{
		InfoCalls:   make([]string, 0),
		DebugCalls:  make([]string, 0),
		WarnCalls:   make([]string, 0),
		ErrorCalls:  make([]string, 0),
		InfofCalls:  make([]string, 0),
		DebugfCalls: make([]string, 0),
		WarnfCalls:  make([]string, 0),
		ErrorfCalls: make([]string, 0),
	}
}

func stringInList(str string, list []string) bool {
	for _, item := range list {
		if str == item {
			return true
		}
	}
	return false
}
