package saltlog

import (
	"strings"
	"testing"
)

func TestTestLogger(t *testing.T) {
	expectedLogs := map[int][]string{
		LevelInfo: []string{
			"test info",
			"test infoarg",
		},
		LevelDebug: []string{
			"test debug",
			"test debugarg",
		},
		LevelWarning: []string{
			"test warn",
			"test warnarg",
		},
		LevelError: []string{
			"test error",
			"test errorarg",
		},
	}

	logger := NewTestLogger()
	logger.Info(expectedLogs[LevelInfo][0])
	logger.Debug(expectedLogs[LevelDebug][0])
	logger.Warn(expectedLogs[LevelWarning][0])
	logger.Error(expectedLogs[LevelError][0])
	logger.Infof(expectedLogs[LevelInfo][0]+"%s", "arg")
	logger.Debugf(expectedLogs[LevelDebug][0]+"%s", "arg")
	logger.Warnf(expectedLogs[LevelWarning][0]+"%s", "arg")
	logger.Errorf(expectedLogs[LevelError][0]+"%s", "arg")

	for level, expectedLog := range expectedLogs {
		for _, log := range expectedLog {
			if err := logger.AssertCalledWith(level, log); err != nil {
				t.Errorf("%v", err)
			}
		}
		if err := logger.AssertCalledWith(level, "log not found"); err == nil {
			t.Errorf("Expected 'log not found' to not be present, but it was")
		}
	}

	expectedErr := "Log Level -3 does not exist"
	err := logger.AssertCalledWith(-3, "foo")
	if err.Error() != expectedErr {
		t.Errorf("Expected %s but got %v", expectedErr, err)
	}
}

func TestTestLoggerPanic(t *testing.T) {
	expected := "testpanic"
	defer func() {
		err := recover()
		if err == nil {
			t.Fatalf("expected to panic but it did not")
		}

		actual, _ := err.(string)
		if !strings.Contains(actual, expected) {
			t.Errorf("expected: %s, but got: %s", expected, actual)
		}
	}()

	logger := NewTestLogger()
	logger.Panic(expected)
}

func TestTestLoggerPanicf(t *testing.T) {
	expected := "test panic"
	defer func() {
		err := recover()
		if err == nil {
			t.Fatalf("expected to panic but it did not")
		}

		actual, _ := err.(string)
		if !strings.Contains(actual, expected) {
			t.Errorf("expected: %s, but got: %s", expected, actual)
		}
	}()

	logger := NewTestLogger()
	logger.Panicf("test %s", "panic")
}
