package server

import (
	"context"
	"fmt"
	"io"
	"net/http"
	"testing"
	"time"

	"gitlab.greeson.xyz/highsaltlevels/go-common/saltlog"
)

func doGet(url string, retries int) (string, error) {
	var resp *http.Response
	var err error
	for retry := 0; retry < retries; retry++ {
		resp, err = http.Get(url)
		if err != nil {
			saltlog.Warnf("test", "Failed to GET: %v", err)
			time.Sleep(1)
			continue
		}
		defer resp.Body.Close()
		body, err := io.ReadAll(resp.Body)
		if err != nil {
			return "", err
		}

		return string(body[:]), nil
	}

	return "", err
}

func TestHealthServer(t *testing.T) {
	ctx, cancel := context.WithCancel(context.Background())
	defer cancel()

	port := 20000
	go StartHttpHealthServer(port, ctx, saltlog.LevelDebug)

	data, err := doGet(fmt.Sprintf("http://localhost:%d/health", port), 5)
	if err != nil {
		t.Fatalf("unexpected error %v", err)
	}
	if data != "ok\n" {
		t.Errorf("expected ok response but got %s", data)
	}
}
